#include <stdio.h>
#include <stdlib.h>

#define VPCALC_SECTION	"DEV"
#define VPCALC_SUBSECT	"T6A04"

#include "../config.h"
#include "../include/types.h"
#include "../include/debug.h"

#include "../include/mem.h"
#include "../include/gfx.h"

#include "t6a04.h"

#define LEFT	0x00
#define RIGHT	0x01
#define UP	0x02
#define DOWN	0x03

typedef struct _t6a04_o {
	int active;
	int word8, waste, increment;
	uint x, y;

	gfx_o gfx;
} _t6a04_o;

t6a04_o t6a04_new(gfx_o gfx)
{
	_t6a04_o *new;

	new = (_t6a04_o *)malloc(sizeof(_t6a04_o));

	if (new != NULL) {
		memset(new, 0, sizeof(_t6a04_o));

		new->gfx = gfx;
	}

	return (t6a04_o)new;
}

void t6a04_del(t6a04_o t6a04)
{
	_t6a04_o *_t6a04 = (_t6a04_o *)t6a04;

	free(_t6a04);
}

void t6a04_write_cmd(t6a04_o t6a04, uint data)
{
	_t6a04_o *_t6a04 = (_t6a04_o *)t6a04;

	if ((data & 0xc0) == 0xc0) {
		gfx_set_contrast(_t6a04->gfx, data & 0x3f);
		return;
	}

	if((data < 0x80) && (data >= 0x40)) {
		gfx_set_screen_addr(_t6a04->gfx, data - 0x40);
		return;
	}

	if((data < 0xc0) && (data >= 0x80)) {
		_t6a04->waste = 1;
		_t6a04->y = data - 0x80;
		return;
	}

	if ((data & 0xf0) == 0x20) {
		_t6a04->waste = 1;
		_t6a04->x = data & 0x1f;
		return;
	}

	switch (data) {
	case 0x00:
		_t6a04->word8 = 0x00;
		return;
	case 0x01:
		_t6a04->word8 = 0x20;
		return;
	case 0x02:
		_t6a04->active = 0x00;
		return;
	case 0x03:
		_t6a04->active = 0x10;
		return;
	case 0x04:
		_t6a04->waste = 1;
		_t6a04->increment = UP;
		return;
	case 0x05:
		_t6a04->waste = 1;
		_t6a04->increment = DOWN;
		return;
	case 0x06:
		_t6a04->waste = 1;
		_t6a04->increment = LEFT;
		return;
	case 0x07:
		_t6a04->waste = 1;
		_t6a04->increment = RIGHT;
		return;
	}
}

uint t6a04_read_cmd(t6a04_o t6a04)
{
	_t6a04_o *_t6a04 = (_t6a04_o *)t6a04;

	return (_t6a04->word8 | _t6a04->increment | _t6a04->active);
}

INTERNAL void _increment(_t6a04_o *_t6a04)
{
	if (_t6a04->increment == LEFT)
		_t6a04->x = (_t6a04->x - 1) & 0x0f;
	else if (_t6a04->increment == RIGHT)
		_t6a04->x = (_t6a04->x + 1) & 0x0f;
	else if (_t6a04->increment == UP)
		_t6a04->y = (_t6a04->y - 1) & 0x3f;
	else if (_t6a04->increment == DOWN)
		_t6a04->y = (_t6a04->y + 1) & 0x3f;
	else
		_err_report("Invalid increment!");
}

void t6a04_write_data(t6a04_o t6a04, uint data)
{
	_t6a04_o *_t6a04 = (_t6a04_o *)t6a04;
	uint8 btmp;
	uint16 wtmp, mask, addr;

	if (_t6a04->word8) {
		addr = (_t6a04->y * 12) + _t6a04->x;

		gfx_write(_t6a04->gfx, addr, data);
	} else {
		btmp = _t6a04->x * 6;
		addr = (_t6a04->y * 12) + (btmp / 8);
		btmp = 10 - (btmp & 0x07);
		mask = 0x3f;

		data = (data & 0x3f) << btmp;
		mask <<= btmp;

		wtmp = ((uint)gfx_read(_t6a04->gfx, addr) << 8) |      
			(uint)gfx_read(_t6a04->gfx, addr+1);

		wtmp = (wtmp & ~mask) | data;

		gfx_write(_t6a04->gfx, addr, (wtmp >> 8));
		gfx_write(_t6a04->gfx, addr+1, wtmp & 0xff);
	}

	_increment(_t6a04);
}

uint t6a04_read_data(t6a04_o t6a04)
{
	_t6a04_o *_t6a04 = (_t6a04_o *)t6a04;
	uint8 btmp;
	uint16 addr;

	if (_t6a04->waste) {
		_t6a04->waste = 0;
		return 0xff;
	} else if (_t6a04->word8) {
		addr = (_t6a04->y * 12) + _t6a04->x;

		_increment(_t6a04);
		return gfx_read(_t6a04->gfx, addr);
	} else {
		addr = _t6a04->x * 6;
		btmp = 10 - (addr & 0x07);

		addr = (_t6a04->y * 12) + (addr / 8);

		btmp = (((uint)gfx_read(_t6a04->gfx, addr) << 8) |
			(uint)gfx_read(_t6a04->gfx, addr+1)) >> btmp;

		_increment(_t6a04);
		return btmp & 0x3f;
	}
}

