#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define VPCALC_SECTION	"MEM"
#define VPCALC_SUBSECT	"RAW8"

#include "../config.h"
#include "../include/types.h"
#include "../include/debug.h"

#include "../include/memory.h"

typedef struct _memory_o {
	uint8 memory[0x10000];
} _memory_o;

memory_o memory_new(void)
{
	_memory_o *new;

	new = (_memory_o *) malloc(sizeof(_memory_o));

	if (new != NULL) {
		memset(new, 0, sizeof(_memory_o));
	}

	return (memory_o)new;
}

void memory_del(memory_o memory)
{
	free(memory);
}

size_t memory_size(memory_o memory)
{
	_memory_o *_memory = (_memory_o *)memory;

	return sizeof(_memory->memory); 
}

size_t memory_bus_size(memory_o memory)
{
	return 8;
}

size_t memory_addr_size(memory_o memory)
{
	return 16;
}

void memory_putimage(memory_o memory, void *data)
{
	_memory_o *_memory = (_memory_o *)memory;

	memcpy(_memory->memory, data, sizeof(_memory->memory));
}

void memory_getimage(memory_o memory, void *data)
{
	_memory_o *_memory = (_memory_o *)memory;

	memcpy(data, _memory->memory, sizeof(_memory->memory));
}

uint memory_fetch(memory_o memory, uint addr)
{
	_memory_o *_memory = (_memory_o *)memory;

	addr = addr & 0xffff;

	return _memory->memory[addr];
}

uint memory_read(memory_o memory, uint addr)
{
	_memory_o *_memory = (_memory_o *)memory;

	addr = addr & 0xffff;

	return _memory->memory[addr];
}

void memory_write(memory_o memory, uint addr, uint data)
{
	_memory_o *_memory = (_memory_o *)memory;

	addr = addr & 0xffff;

	_memory->memory[addr] = data;
}

uint8 memory_fetch_byte(memory_o memory, uint addr)
{
	_memory_o *_memory = (_memory_o *)memory;

	addr &= 0xffff;

	return _memory->memory[addr];
}

uint8 memory_read_byte(memory_o memory, uint addr)
{
	_memory_o *_memory = (_memory_o *)memory;

	addr &= 0xffff;

	return _memory->memory[addr];
}

void memory_write_byte(memory_o memory, uint addr, uint8 data)
{
	_memory_o *_memory = (_memory_o *)memory;

	addr &= 0xffff;

	_memory->memory[addr] = data;
}

void memory_remap(memory_o memory, uint page, uint addr, uint perm)
{
	/* This function intentionally left blank. */
	return;
}

int memory_raw(memory_o memory, void **rd_ptr, void **wr_ptr, void **ex_ptr)
{
	_memory_o *_memory = (_memory_o *)memory;

	*rd_ptr = (void *)_memory->memory;
	*wr_ptr = (void *)_memory->memory;
	*ex_ptr = (void *)_memory->memory;

	return 1;
}
